package net.sf.javaprinciples.data.transformer;

import java.util.Collection;

import net.sf.javaprinciples.core.UnexpectedException;

/**
 * Mapper to handle the processing of an existing Collection of Objects.
 * Expects an output of type Collection.
 * Processes through that Collection to pass the existing instances to its mappers.
 *
 * @author Kay Chevalier
 */
public class CollectionOutputMapper<T,U> extends ModelElementMapper implements Mapper<T, U>
{

    @Override
    public void map(T input, U output)
    {
        Object outputAttribute = null;
        if(getDestinationModelElement() != null)
        {
            String attributeName =  ModelElementMapperHelper.getPropertyNameFromElement(getDestinationModelElement());
            outputAttribute = objectTypeMapper.getAttributeFromObject(output, attributeName);
        }

        if (!(outputAttribute instanceof Collection<?>))
        {
            throw new UnexpectedException("CollectionOutputMapper expects an ArrayList for the output");
        }

        Collection outputCollection = (Collection)outputAttribute;
        for (Object outputInstance : outputCollection)
        {

                processMapperAssociations(input, outputInstance);
        }
    }
}