package net.sf.javaprinciples.data.transformer;

import net.sf.jcc.model.parser.uml2.ElementStore;
import net.sf.jcc.model.parser.uml2.ModelElement;

/**
 * Inject the remaining attributes of a ComplexMapper.
 *
 * @author Warwick Slade
 */
public class ComplexMapperBuilder<T,U> extends ModelElementFactory implements ModelElementMapperBuilder<ModelElementMapper>
{
    private MapperFactory mapperFactory;

    @Override
    public void build(ModelElementMapper mapperModelElement, TransformerFactory.DIRECTION_TYPE direction)
    {
        ElementStore store = mapperModelElement.store;
        ModelElement modelElement = mapperModelElement.getModelElement();

        mapperModelElement.setModelElementMappers(retrieveMappersFromAssociations(modelElement, store, direction));
        mapperModelElement.setPostProcessorMappers(retrievePostProcessorFromAssociations(modelElement, store, direction));
        mapperModelElement.setPreProcessorMappers(retrievePreProcessorFromAssociations(modelElement, store, direction));

        ModelElement sourceModelElement = mapperModelElement.getSourceModelElement();
        ModelElement destinationModelElement = mapperModelElement.getDestinationModelElement();

        log.info("Creating Mapper: {}, {}, Source Model Element: {}, Target Model Element: {} ",
                new Object[] {
                        mapperModelElement.getClass().getName(),
                        mapperModelElement.getModelElement().getGuid(),
                        sourceModelElement == null ? null : sourceModelElement.getGuid(),
                        destinationModelElement == null ? null : destinationModelElement.getGuid()});

    }

    protected ModelElementMapper<T,U>  createModelElementMapper(String mapperGuid, ElementStore store,
                                                                TransformerFactory.DIRECTION_TYPE direction)
    {
        return (ModelElementMapper<T,U>)mapperFactory.createMapper(mapperGuid, store, direction);
    }

    public void setMapperFactory(MapperFactory mapperFactory)
    {
        this.mapperFactory = mapperFactory;
    }
}
