package net.sf.javaprinciples.data.transformer;

import org.apache.commons.lang.StringUtils;

import net.sf.jcc.model.parser.uml2.ElementStore;
import net.sf.jcc.model.parser.uml2.ModelElement;

/**
 * Inject the remainder of the dependencies for an AttributeMapper.
 *
 * 1. Construct the set of individual mappings
 *
 * @author Warwick Slade
 */
public class DefaultAttributeMapperBuilder<M extends DefaultAttributeMapper, T,U> extends ModelElementFactory implements ModelElementMapperBuilder<M>
{
    private static final String RIGHT_ATTRIBUTE_MAPPER_TAG = "rightAttributeMapper";
    private static final String LEFT_ATTRIBUTE_MAPPER_TAG = "leftAttributeMapper";
    private static final String DEFAULT_SOURCE_ATTRIBUTE_MAPPER = "defaultSourceAttributeMapper";

    private SourceDestinationAttributeMapperBeanFactory sourceDestinationAttributeMapperBeanFactory;

    @Override
    public void build(M mapper , TransformerFactory.DIRECTION_TYPE direction)
    {
        ModelElement mapperModelElement = mapper.getModelElement();

//        if (mapper.getDestinationModelElements().isEmpty())
//        {
//            throw new UnexpectedException("You must have at least one target model element for "
//                    + mapperModelElement.getGuid());
//        }

        ModelElement parentModelElement = mapper.store.get(mapperModelElement.getParentGuid());

        String leftAttributeMapperDefault = ModelElementMapperHelper.retrieveModelElementTagValue(parentModelElement, determineDirection(true, direction));
        ModelElementSourceDestinationMapper sourceAttributeMapper = getModelElementSourceDestinationAttributeMapper(
                mapperModelElement, determineDirection(true, direction), leftAttributeMapperDefault);
        mapper.setSourceModelElementMapper(sourceAttributeMapper);

        String rightAttributeMapperDefault = ModelElementMapperHelper.retrieveModelElementTagValue(parentModelElement, determineDirection(false, direction));
        ModelElementSourceDestinationMapper destinationAttributeMapper = getModelElementSourceDestinationAttributeMapper(
                mapperModelElement, determineDirection(false, direction), rightAttributeMapperDefault);
        mapper.setDestinationModelElementMapper(destinationAttributeMapper);
    }

    protected String determineDirection(boolean isSource, TransformerFactory.DIRECTION_TYPE direction)
    {
        if (isSource)
        {
            if (TransformerFactory.DIRECTION_TYPE.leftToRight.equals(direction))
            {
                return LEFT_ATTRIBUTE_MAPPER_TAG;
            }
            else
            {
                return RIGHT_ATTRIBUTE_MAPPER_TAG;
            }
        }
        else
        {
            if (TransformerFactory.DIRECTION_TYPE.leftToRight.equals(direction))
            {
                return RIGHT_ATTRIBUTE_MAPPER_TAG;
            }
            else
            {
                return LEFT_ATTRIBUTE_MAPPER_TAG;
            }
        }
    }

    protected ModelElementSourceDestinationMapper getModelElementSourceDestinationAttributeMapper(ModelElement modelElement,
                                                                                                  String direction,
                                                                                                  String defaultSourceDestinationAttributeMapper)
    {
        String sourceDestinationAttributeMapper = ModelElementMapperHelper.retrieveModelElementTagValue(modelElement, direction);

        AbstractSourceDestinationAttributeMapper sourceDestinationMapper;
        if (sourceDestinationAttributeMapper == null)
        {
            sourceDestinationAttributeMapper = StringUtils.isNotBlank(defaultSourceDestinationAttributeMapper) ?
                    defaultSourceDestinationAttributeMapper : DEFAULT_SOURCE_ATTRIBUTE_MAPPER;
        }

        sourceDestinationMapper = (AbstractSourceDestinationAttributeMapper)sourceDestinationAttributeMapperBeanFactory.createModelElementMapper(sourceDestinationAttributeMapper);

        sourceDestinationMapper.setObjectTypeMapper(objectTypeMapper);
        sourceDestinationMapper.setMappingElement(modelElement);

        return sourceDestinationMapper;
    }

    protected ModelElementMapper<T,U>  createModelElementMapper(String mapperGuid, ElementStore store,
                                                                TransformerFactory.DIRECTION_TYPE direction)
    {
        throw new UnsupportedOperationException();
    }

    public void setSourceDestinationAttributeMapperBeanFactory(SourceDestinationAttributeMapperBeanFactory sourceDestinationAttributeMapperBeanFactory)
    {
        this.sourceDestinationAttributeMapperBeanFactory = sourceDestinationAttributeMapperBeanFactory;
    }


}
