package net.sf.javaprinciples.data.transformer;

import net.sf.jcc.model.parser.uml2.ModelElement;

/**
 * Transformer to transform and map from an input object to an output object.
 * Uses a source and destination model element to determine input and output
 * and model element mappers to delegate mapping functionality to.
 *
 * @author Kay Chevalier
 */
public class MapperTransformer<Input, Output> extends ComplexMapper<Input, Output> implements Transformer<Input, Output>
{
    public Output transform(Input input)
    {
        Output output = instantiateOutput(getDestinationModelElement());
        super.map(input, output);

        return output;
    }

    private Output instantiateOutput(ModelElement destinationModelElement)
    {
        return (Output) ModelElementMapperHelper.createObjectFromClassModelElement(destinationModelElement, store, objectTypeMapper);
    }

    @Override
    protected Input retrieveInput(ModelElement sourceModelElement, Input input)
    {
        return input;
    }


    @Override
    protected Output instantiateOutputFromAttributeName(Output output, String attributeName)
    {
        return output;
    }

    @Override
    protected Output getAttributeFromObject(Output output, String attributeName)
    {
        return output;
    }

    @Override
    protected void assignAttributeToObject(Output destinationOutput, Output output, String attributeName)
    {

    }
}
