package net.sf.javaprinciples.data.transformer;

import java.util.List;

import net.sf.jcc.model.parser.uml2.ModelElement;

/**
 * Implementation of a ModelElementSourceDestinationMapper based on the mapping element.
 * Does not consider the passed in modelElements.
 *
 * @author Warwick
 */
public class PassThroughSourceAttributeMapper extends AbstractSourceDestinationAttributeMapper
{
    /**
     * Given a list of {@link net.sf.jcc.model.parser.uml2.ModelElement} objects, retrieves the element name
     * from the first model element in the list.
     *
     * @param modelElements - a list of ModelElements to retrieve the attribute name from.
     * @return - the attribute name
     */
    @Override
    public String getAttributeName(List<ModelElement> modelElements)
    {
        return ModelElementMapperHelper.getPropertyNameFromElement(mappingElement);
    }

    @Override
    public String getAttributeName(ModelElement modelElement)
    {
        return ModelElementMapperHelper.getPropertyNameFromElement(mappingElement);
    }

    /**
     * Given a list of {@link net.sf.jcc.model.parser.uml2.ModelElement} objects, retrieves the attribute name from the
     * first model element in the list. This attribute name is then used to find the
     * attribute of the parent object in which the sourceAttributeValue should be set.
     *
     * @param modelElements - a list of ModelElements to retrieve the parent object attribute name from.
     * @param sourceAttributeValue - the value to be set on the parent object
     * @param parentObject - the object to set the attribute value on.
     */
    @Override
    public void setAttributeOnParentObject(List<ModelElement> modelElements, Object sourceAttributeValue, Object parentObject)
    {
        throw new UnsupportedOperationException();
    }

    /**
     * Given a list of {@link net.sf.jcc.model.parser.uml2.ModelElement} objects, retrieves the attribute name from the
     * first model element in the list. This attribute name is then used to find the corresponding
     * attribute value from the parent object.
     * @param modelElements - a list of ModelElements to retrieve the parent object attribute name from.
     * @param parentObject - the object to retrieve the attribute value from.
     * @return - the attribute object
     */
    @Override
    public Object getAttributeFromObject(List<ModelElement> modelElements, Object parentObject)
    {
        return parentObject;
    }
}
